//
//  DashboardViewController.m
//  HA7 logger mini05
//
//  Created by 富樫 豊彦 on 2015/03/10.
//  Copyright (c) 2015年 富樫 豊彦. All rights reserved.
//

#include <math.h>
#import "DashboardViewController.h"
#import "AppDelegate.h"
#import <CoreMotion/CoreMotion.h>


@interface DashboardViewController () {
    AppDelegate             *appDeligate;
    ObdDataLogger           *obd;
    NSTimer                 *refreshTimer;
    int                      stopTm;
    enum {INIT,CONNECT,DISCONNECT} connectStat;

    UIImagePickerController *imagePicker;
    BOOL                     recording;
    BOOL                     saving;
    NSTimer                 *intervalTimer;
    NSString                *beforeVideoFile;
    CMMotionManager         *motionManager;
}

@end


@implementation DashboardViewController

//
//  Manage of standard view
//
- (void)viewDidLoad
{
    [super viewDidLoad];

    appDeligate  = [[UIApplication sharedApplication] delegate];
    obd          = appDeligate.obd;
    connectStat  = INIT;
    recording    = NO;
    saving       = NO;
    refreshTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(refreshView:) userInfo:nil repeats:YES];

    [self createImagePicker];
}

- (void)viewWillAppear:(BOOL)animated {
    NSLog(@"viewWillAppear");
    [super viewWillAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated {
    NSLog(@"viewWillDisappear");
    [super viewWillDisappear:animated];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

//
//  Instrument view
//
- (void)refreshView:(NSTimer *)timer
{
//  NSLog(@"refreshView");
    if (obd.valid) {
        if (obd.dist > 0.001) {   // 1[m/s]
            stopTm = 0;
        } else {
            stopTm++;
        }
        if (connectStat != CONNECT) {
            connectStat = CONNECT;
            [statMsg  setText:@"online"];
            mainPanel.alpha = 1.0;
            subPanel.alpha  = 1.0;
            [self startRecord];
        }
    } else {
        if (connectStat != DISCONNECT) {
            connectStat = DISCONNECT;
            [statMsg  setText:@"offline"];
            mainPanel.alpha = 0.3;
            subPanel.alpha  = 0.3;
            [self stopRecord];
        }
    }
    
    [vss        setText:[[NSString alloc] initWithFormat:@"%d", obd.vss]];
    [rpm        setText:[[NSString alloc] initWithFormat:@"%d", obd.rpm]];
    [rpmG       setIntValue:obd.rpm];
    
    [effNow     setText:[[NSString alloc] initWithFormat:@"%.1f", obd.eff]];
    [effDay     setText:[[NSString alloc] initWithFormat:@"%.1f", obd.eff_d]];
    [effAve     setText:[[NSString alloc] initWithFormat:@"%.1f", obd.eff_s]];
    [effNowG    setFloatValue:obd.eff];
    [effDayG    setFloatValue:obd.eff_d];
    [effAveG    setFloatValue:obd.eff_s];
    
    [stopTim    setText:[[NSString alloc] initWithFormat:@"%d:%02d", stopTm/60, stopTm%60]];
    [fuse_d     setText:[[NSString alloc] initWithFormat:@"%.2f", obd.fuse_d]];
    [dist_d     setText:[[NSString alloc] initWithFormat:@"%.2f", obd.dist_d]];
    [runtmDay   setText:[[NSString alloc] initWithFormat:@"%ld:%02ld", obd.runtm_d/60, obd.runtm_d%60]];
    [runtmDayG  setProgress:((double)obd.movtm_d/(double)obd.runtm_d)];
    
    [remainFuel setText:[[NSString alloc] initWithFormat:@"%.2f", obd.remainFuel]];
    [remainFuelG setProgress:(obd.remainFuel/FUEL_TANK)];
    [remainDist setText:[[NSString alloc] initWithFormat:@"%d",(int)(obd.remainFuel * obd.eff_s)]];
    [ect        setText:[[NSString alloc] initWithFormat:@"%d", obd.ect]];
    [ectG       setProgress:((obd.ect + 30.0)/130.0)];
    [vpwr       setText:[[NSString alloc] initWithFormat:@"%.1f",(float)obd.vpwr/1000.0]];
    [vpwrG      setProgress:((obd.vpwr - 11000)/4000.0)];
}

//
//  Camera view
//
- (void)createImagePicker
{
    if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera]) {
        imagePicker = [[UIImagePickerController alloc] init];
        imagePicker.delegate             = self;
        imagePicker.sourceType           = UIImagePickerControllerSourceTypeCamera;
        imagePicker.mediaTypes           = [NSArray arrayWithObject:@"public.movie"];
        imagePicker.cameraCaptureMode    = UIImagePickerControllerCameraCaptureModeVideo;
        imagePicker.allowsEditing        = NO;
        imagePicker.showsCameraControls  = NO;
        imagePicker.cameraViewTransform  = CGAffineTransformIdentity;
        imagePicker.videoQuality         = UIImagePickerControllerQualityTypeIFrame1280x720;
        imagePicker.videoMaximumDuration = 60.0;
        if ([UIImagePickerController isCameraDeviceAvailable: UIImagePickerControllerCameraDeviceRear]) {
            imagePicker.cameraDevice     = UIImagePickerControllerCameraDeviceRear;
        } else {
            imagePicker.cameraDevice     = UIImagePickerControllerCameraDeviceFront;
        }
        if ([UIImagePickerController isFlashAvailableForCameraDevice:imagePicker.cameraDevice]) {
            imagePicker.cameraFlashMode  = UIImagePickerControllerCameraFlashModeOff;
        }
        imagePicker.cameraOverlayView = cameraOverlayView;
        [recordMsg setText:@"Ready"];
    } else {
        [recordMsg setText:@"No camera"];
        recordSw.alpha  = 0.0;
        recordSwL.alpha = 0.0;
        saveSw.alpha    = 0.0;
        saveSwL.alpha   = 0.0;
    }
}

- (void)startRecord
{
    NSLog(@"startRecord");
    if (imagePicker) {
        if (!recording) {
            UIInterfaceOrientation orientation = [[UIApplication sharedApplication] statusBarOrientation];
            CGRect nowRect = [cameraOverlayView frame];         // reset of view position
            [imagePicker.view setFrame:nowRect];
            [self presentViewController:imagePicker animated:YES completion:nil];
            // changing a display direction of overlay view by force
            if (orientation == UIInterfaceOrientationLandscapeLeft) {
                cameraOverlayView.transform = CGAffineTransformMakeRotation(M_PI / -2.0);
            } else if (orientation == UIInterfaceOrientationLandscapeRight) {
                cameraOverlayView.transform = CGAffineTransformMakeRotation(M_PI / 2.0);
            }
            intervalTimer = [NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(waitRecrding:) userInfo:nil repeats:YES];
            recording = YES;
            
            motionManager = [[CMMotionManager alloc] init];
            if ([motionManager isAccelerometerAvailable]) {
                motionManager.accelerometerUpdateInterval = 0.1;    // [sec]
                [motionManager startAccelerometerUpdatesToQueue:[NSOperationQueue mainQueue] withHandler:^(CMAccelerometerData *accelerometerData, NSError *error) {
                    if (accelerometerData.acceleration.z > SAVE_START_G ||
                        accelerometerData.acceleration.z < -SAVE_START_G) {
                        [self startSave];
                    }
                }];
            } else {
                motionManager = nil;
            }
        }
        [recordSw setOn:YES];
    }
}

- (void)waitRecrding:(NSTimer *)timer
{
    NSLog(@"waitRecrding");
    if ([imagePicker startVideoCapture]) {
        [intervalTimer invalidate];
        intervalTimer = [NSTimer scheduledTimerWithTimeInterval:RECORD_PERIOD target:self selector:@selector(changeRecrd:) userInfo:nil repeats:YES];
        [appDeligate setDisplayOn];
        [recordMsg setText:@"REC"];
    } else {
        [recordMsg setText:@"Wait"];
    }
}

- (void)changeRecrd:(NSTimer *)timer
{
    NSLog(@"changeRecrd");
    if (recording) {
        [recordMsg setText:@"Busy"];
        [imagePicker stopVideoCapture];
        if ([imagePicker startVideoCapture]) {
            NSLog(@"Error recovery is success");
        }
    }
}

- (void)stopRecord
{
    NSLog(@"stopRecord");
    if (imagePicker) {
        if (recording) {
            [recordMsg setText:@"STOP"];
            [imagePicker stopVideoCapture];
            [intervalTimer invalidate];
            intervalTimer = nil;
            
            if (motionManager) {
                [motionManager stopAccelerometerUpdates];
            }
        }
        [recordSw setOn:NO];
        [saveSw setOn:NO];
        [appDeligate unsetDisplayOn];
    }
}

- (void)startSave
{
    if (imagePicker) {
        if (!saving) {
            if (beforeVideoFile) {
                UISaveVideoAtPathToSavedPhotosAlbum(beforeVideoFile, self, @selector(video:didFinishSavingWithError:contextInfo:), NULL);
                beforeVideoFile = nil;
            }
            saving = YES;
            [self startRecord];
        }
        [saveSw setOn:YES];
    }
}

- (void)stopSave
{
    if (imagePicker) {
        if (saving) {
            [imagePicker stopVideoCapture];
        }
        [saveSw setOn:NO];
    }
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    NSURL    *videoURL    = [info valueForKey:UIImagePickerControllerMediaURL];
    NSString *pathToVideo = [videoURL path];

    if (beforeVideoFile) {
        NSLog(@"Remove=%@", beforeVideoFile);
        [[NSFileManager defaultManager] removeItemAtPath:beforeVideoFile error:nil];
        [[NSFileManager defaultManager] removeItemAtPath:[beforeVideoFile stringByDeletingLastPathComponent] error:nil];
    }
    
    if (saving) {           // Permanent record
        UISaveVideoAtPathToSavedPhotosAlbum(pathToVideo, self, @selector(video:didFinishSavingWithError:contextInfo:), NULL);
    }

    if (!recordSw.on) {
        cameraOverlayView.transform = CGAffineTransformIdentity;
        [self dismissViewControllerAnimated:YES completion:nil];
        [recordMsg setText:@"Ready"];
        recording = NO;
    }

    if (recording) {
        if ([imagePicker startVideoCapture]) {
            NSLog(@"RESTART OK");
            [recordMsg setText:@"REC"];
        } else {
            NSLog(@"RESTART ABORT");
            [recordMsg setText:@"ERR"];
        }
    }

    beforeVideoFile = pathToVideo;
}

- (void)video:(NSString *)videoPath didFinishSavingWithError:(NSError *)error contextInfo:(void *)contextInfo {
    NSLog(@"Movie saved & remove: %@", videoPath);
    [[NSFileManager defaultManager] removeItemAtPath:videoPath error:nil];
    [[NSFileManager defaultManager] removeItemAtPath:[videoPath stringByDeletingLastPathComponent] error:nil];
    if (!saveSw.on) {
        saving = NO;
    }
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    NSLog(@"imagePickerControllerDidCancel");
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (IBAction)slidedRecordSw:(id)sender{
    NSLog(@"slidedRecordSw");
    if (recordSw.on) {
        [self startRecord];
    } else {
        [self stopRecord];
    }
}

- (IBAction)slidedSaveSw:(id)sender{
    NSLog(@"slidedSaveSw");
    if (saveSw.on) {
        [self startSave];
    } else {
        [self stopSave];
    }
}

- (IBAction)exit1:(id)sender
{
    if (refreshTimer) {
        [refreshTimer invalidate];
        refreshTimer = nil;
    }
    if (saving) {
        [self stopSave];
    } else if (recording) {
        [self stopRecord];
    } else {
        if (beforeVideoFile) {
            NSLog(@"Remove=%@", beforeVideoFile);
            [[NSFileManager defaultManager] removeItemAtPath:beforeVideoFile error:nil];
            [[NSFileManager defaultManager] removeItemAtPath:[beforeVideoFile stringByDeletingLastPathComponent] error:nil];
            beforeVideoFile = nil;
        }
        [self dismissViewControllerAnimated:YES completion:nil];
        UINavigationController *n = [self navigationController];
        [n popViewControllerAnimated:YES];
    }
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
