//
//  ObdDataLogger.h
//  HA7 logger mini
//  Copyright(C)2012-2015 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2013.04.30 v0.0  First cording
//    2013.05.04 v0.4  Commit
//    2015.03.10 v0.5  Adds heading
//
#import <Foundation/Foundation.h>
//#import <CoreLocation/CoreLocation.h>                              DEL 2015.03.10

//@interface ObdDataLogger : NSObject <CLLocationManagerDelegate>    DEL 2015.03.10
@interface ObdDataLogger : NSObject                              //  ADD 2015.03.10

// Origin information
@property (nonatomic) int    ect;        // Engine Coolant Temperature -40..215["C]
@property (nonatomic) int    shrtft1;    // Short Term Fuel Trim - Bank 1 -100..99[%]
@property (nonatomic) int    longft1;    // Long Term Fuel Trim - Bank 1 -100..99[%]
@property (nonatomic) int    map;        // Intake Manifold Absolute Pressure 0..255[kPa]
@property (nonatomic) int    rpm;        // Engine RPM 0..16383[min^-1]
@property (nonatomic) int    vss;        // Vehicle Speed Sensor 0..255[km/h]
@property (nonatomic) int    sparkadv;   // Ignition Timing Advance for #1 Cylinder -64..63["]
@property (nonatomic) int    iat;        // Intake Air Temperature -40..215["C]
@property (nonatomic) int    tp;         // Absolute Throttle Position 0..100[%]
@property (nonatomic) int    o2s11;      // Bank 1 Sensor 1 - Oxygen Sensor Output Voltage 0..1275[mV]
@property (nonatomic) int    shrtft11;   // Bank 1 Sensor 1 - Short Term Fuel Trim -100..99[%]
@property (nonatomic) int    vpwr;       // Control module voltage 0..65535[mV]
//@property (nonatomic) CLLocation   *nowLoc;     // GPS                DEL 2015.03.10
@property (nonatomic) double latitude;   // Location 0..360["]          ADD 2015.03.10
@property (nonatomic) double longitude;  // Location 0..360["]          ADD 2015.03.10
@property (nonatomic) int    altitude;   // Location [m]                ADD 2015.03.10
@property (nonatomic) int    heading;    // Magnetic north -180..180["] ADD 2015.03.10
// Calculated value
@property (nonatomic) float  af;         // Air flow on intake manifold [g/sec]
@property (nonatomic) float  ff;         // Fuel flow on intake manifold [g/sec]
@property (nonatomic) float  r;          // Air density [kg/m^3]
@property (nonatomic) float  v;          // Air volume [m^3/sec]
@property (nonatomic) float  atm;        // Air pressure [atm]
@property (nonatomic) float  dist;       // Interval distance [km]
@property (nonatomic) float  fuse;       // Interval used fuel [L]
@property (nonatomic) float  eff;        // Realtime efficient [km/L]
// Accumurated value
@property (nonatomic) double dist_s;     // Accumurated distance [km]
@property (nonatomic) double fuse_s;     // Accumurated used fuel [L]
@property (nonatomic) float  eff_s;      // Average efficient [km/L]
@property (nonatomic) long   runtm;      // Accumurated engine run time [sec]
@property (nonatomic) long   movtm;      // Accumurated vehicle moving time [sec]

@property (nonatomic) double dist_d;     // Distance [km/day]
@property (nonatomic) double fuse_d;     // Used fuel [L/day]
@property (nonatomic) float  eff_d;      // Average efficient [km/L]
@property (nonatomic) long   runtm_d;    // Run time [sec/day]
@property (nonatomic) long   movtm_d;    // Move time [sec/day]

@property (nonatomic) double remainFuel; // Fuel tank capacity[L]

// Status
@property (nonatomic) BOOL   valid;      //
@property (nonatomic) BOOL   valid_l;    // ADD 2015.03.10
@property (nonatomic) BOOL   valid_h;    // ADD 2015.03.10

// Calculate constnt
#define INTVL 1.0                        // Log interval [s]
#define C_VOL 0.000656					 // Cylnder volume [m^3]
#define E_CYC 2.0						 // Engine output shaft rotation / Cycle

@end
