//
//  RouteMapViewController.m
//  HA7 logger mini05
//
//  Created by 富樫 豊彦 on 2015/03/09.
//  Copyright (c) 2015年 富樫 豊彦. All rights reserved.
//

#import "RouteMapViewController.h"

#define MIN_DIST    0.0001          // ~=10m
#define SKIP_COUNT  10              // ~=10s
#define MAX_POINT   360             // ~=1h

@interface RouteMapViewController () <MKMapViewDelegate>

@end

@implementation RouteMapViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.map.delegate = self;
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}


- (void)viewWillAppear:(BOOL)animated {
    // Date
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateStyle:NSDateFormatterMediumStyle];
    [self.deb setText:[dateFormatter stringFromDate:self.showDate]];
    [self setTitle:[dateFormatter stringFromDate:self.showDate]];

    // Map
    NSDateComponents *dc = [[NSCalendar currentCalendar] components:NSCalendarUnitYear|NSCalendarUnitMonth|NSCalendarUnitDay fromDate:self.showDate];
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path1 = [[NSString alloc] initWithFormat:@"%@/%d-%02d-%02d.csv", [paths objectAtIndex:0], (int)dc.year, (int)dc.month, (int)dc.day];
    const char *fname = [path1 UTF8String];
    FILE *fp = fopen(fname,"r");
    if (fp != NULL) {
        CLLocationCoordinate2D points[MAX_POINT];
        int     pintNum;
        int     sc;
        double  c1;
        double  c2;
        double  c1_max;
        double  c2_max;
        double  c1_min;
        double  c2_min;
        char    buff[300];
        
        pintNum = 0;
        c1      = 0.0;
        c2      = 0.0;
        sc      = 1;        // Header line
        c1_max  = 0.0;
        c2_max  = 0.0;
        c1_min  = 0.0;
        c2_min  = 0.0;
        while(fgets(buff, sizeof(buff), fp) != NULL) {
            char    *k;
            char    *kk;
            int     i;
            double  c11;
            double  c22;
            
            if (--sc >= 0) {
                continue;
            }
            c11 = 0.0;
            c22 = 0.0;
            for(k = strtok_r(buff, ",", &kk), i = 0; k != NULL; k = strtok_r(NULL, ",", &kk), i++) {
                if (i == 20) {
                    c11 = atof(k);
                } else if (i == 21) {
                    c22 = atof(k);
                }
            }
            if ((c11 == 0.0) || (c22 == 0.0)) { // Data error
                continue;
            }
            if (sqrt(pow(c11 - c1, 2.0) + pow(c22 - c2, 2.0)) > MIN_DIST) {
                c1 = c11;
                c2 = c22;
                points[pintNum].latitude  = c1;
                points[pintNum].longitude = c2;
                if (pintNum == 0) {
                    c1_max = c1;
                    c2_max = c2;
                    c1_min = c1;
                    c2_min = c2;
                } else {
                    c1_max = MAX(c1_max, c1);
                    c2_max = MAX(c2_max, c2);
                    c2_min = MIN(c2_min, c2);
                    c1_min = MIN(c1_min, c1);
                }
                if (++pintNum >= MAX_POINT) {
                    break;
                }
            }
            sc = SKIP_COUNT;
        }
        fclose(fp);
        
        if (pintNum > 1) {
            MKPolyline * poly = [MKPolyline polylineWithCoordinates:points count:pintNum];
            poly.title = @"Route";
            [self.map addOverlay:poly];
            
            MKCoordinateRegion theRegion = [self.map region];
            theRegion.center.latitude     = (c1_max + c1_min) / 2.0;
            theRegion.center.longitude    = (c2_max + c2_min) / 2.0;
            theRegion.span.latitudeDelta  = (c1_max - c1_min) * 1.2;
            theRegion.span.longitudeDelta = (c2_max - c2_min) * 1.2;
            [self.map setRegion:theRegion animated:YES];
        }
    }
}


#pragma mark - Map View Delegate


- (MKOverlayView *)mapView:(MKMapView *)mapView viewForOverlay:(id <MKOverlay>)overlay
{
    if ([overlay isKindOfClass:[MKPolyline class]]) {
        MKPolylineView*    aView = [[MKPolylineView alloc] initWithPolyline:(MKPolyline *)overlay];
        aView.lineWidth   = 5;
        if (aView.polyline.pointCount < MAX_POINT) {
            aView.strokeColor = [UIColor blueColor];
        } else {
            aView.strokeColor = [UIColor orangeColor];
        }
        return aView;
    }
    return nil;
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
